@@ -32,19 +32,20 @@ import six
 
 from keystoneauth1 import access
 from keystoneauth1 import adapter
-from keystoneauth1.identity import base
 from keystoneauth1 import discover
+from keystoneauth1.identity import base
 from oslo_utils import encodeutils
 from oslo_utils import importutils
 from oslo_utils import strutils
 osprofiler_web = importutils.try_import("osprofiler.web")  # noqa
 import requests
+from six.moves import urllib
 import six.moves.urllib.parse as urlparse
 
+from cinderclient._i18n import _
 from cinderclient import api_versions
 from cinderclient import exceptions
 import cinderclient.extension
-from cinderclient._i18n import _
 
 try:
     from eventlet import sleep
@@ -83,8 +84,29 @@ def get_server_version(url):
 
     logger = logging.getLogger(__name__)
     try:
-        scheme, netloc, path, query, frag = urlparse.urlsplit(url)
-        response = requests.get(scheme + '://' + netloc)
+        u = urllib.parse.urlparse(url)
+        version_url = None
+
+        # NOTE(andreykurilin): endpoint URL has at least 2 formats:
+        #   1. The classic (legacy) endpoint:
+        #       http://{host}:{optional_port}/v{1 or 2 or 3}/{project-id}
+        #       http://{host}:{optional_port}/v{1 or 2 or 3}
+        #   3. Under wsgi:
+        #       http://{host}:{optional_port}/volume/v{1 or 2 or 3}
+        for ver in ['v1', 'v2', 'v3']:
+            if u.path.endswith(ver) or "/{0}/".format(ver) in u.path:
+                path = u.path[:u.path.rfind(ver)]
+                version_url = '%s://%s%s' % (u.scheme, u.netloc, path)
+                break
+
+        if not version_url:
+            # NOTE(andreykurilin): probably, it is one of the next cases:
+            #  * https://volume.example.com/
+            #  * https://example.com/volume
+            # leave as is without cropping.
+            version_url = url
+
+        response = requests.get(version_url)
         data = json.loads(response.text)
         versions = data['versions']
         for version in versions:
@@ -132,15 +154,16 @@ def _log_request_id(logger, resp, service_name):
 class SessionClient(adapter.LegacyJsonAdapter):
 
     def __init__(self, *args, **kwargs):
-        self.api_version = kwargs.pop('api_version', None)
-        self.api_version = self.api_version or api_versions.APIVersion()
+        apiver = kwargs.pop('api_version', None) or api_versions.APIVersion()
+        if not isinstance(apiver, api_versions.APIVersion):
+            apiver = api_versions.APIVersion(str(apiver))
+        if apiver.ver_minor != 0:
+            kwargs['default_microversion'] = apiver.get_string()
         self.retries = kwargs.pop('retries', 0)
         self._logger = logging.getLogger(__name__)
         super(SessionClient, self).__init__(*args, **kwargs)
 
     def request(self, *args, **kwargs):
-        kwargs.setdefault('headers', kwargs.get('headers', {}))
-        api_versions.update_headers(kwargs["headers"], self.api_version)
         kwargs.setdefault('authenticated', False)
 
         # Note(tpatil): The standard call raises errors from
@@ -195,7 +218,7 @@ class SessionClient(adapter.LegacyJsonAdapter):
             version = get_volume_api_from_url(self.get_endpoint())
         except exceptions.UnsupportedVersion as e:
             msg = (_("Service catalog returned invalid url.\n"
-                     "%s") % six.text_type(e.message))
+                     "%s") % six.text_type(e))
             raise exceptions.UnsupportedVersion(msg)
 
         return version
@@ -446,10 +469,10 @@ class HTTPClient(object):
             if self.management_url == self.bypass_url:
                 msg = (_("Invalid url was specified in --os-endpoint or "
                          "environment variable CINDERCLIENT_BYPASS_URL.\n"
-                         "%s") % six.text_type(e.message))
+                         "%s") % six.text_type(e))
             else:
                 msg = (_("Service catalog returned invalid url.\n"
-                         "%s") % six.text_type(e.message))
+                         "%s") % six.text_type(e))
 
             raise exceptions.UnsupportedVersion(msg)
 
